local RLR_LONG_ADDON_NAME_PREFIX = '<RaidLootRoller> '
local RLR_SHORT_ADDON_NAME_PREFIX = '<RLR> '

local function CanUseRaidWarning()
	return UnitIsGroupLeader('player') or UnitIsGroupAssistant('player')
end

local function GetBroadcastChannel(isHighPriority)
	local channel
	if IsInGroup() then
		if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
			if CanUseRaidWarning() and isHighPriority then 
				channel = 'RAID_WARNING' -- RAID_WARNING
			else
				channel = 'INSTANCE_CHAT' -- INSTANCE_CHAT
			end
		elseif IsInRaid() then
			if CanUseRaidWarning() and isHighPriority then 
				channel = 'RAID_WARNING' -- RAID_WARNING
			else
				channel = 'RAID' -- RAID
			end
		else	
			channel = 'PARTY' -- PARTY
		end
	else
		channel = 'EMOTE'  -- for testing purposes
	end
	return channel
end

function RLR_GetFullName(name, realm)
	if name == nil then
		return nil
	elseif realm == nil then
		return name
	else
		return name .. '-' .. realm
	end
end

function RLR_GetUnitNameWithRealm(unit)
	local guid
	if unit ~= nil then
		guid = UnitGUID(unit)
	end
	if guid ~= nil then
		local _, _, _, _, _, name, realm = GetPlayerInfoByGUID(guid)
		if not realm or realm == '' then
			realm = GetRealmName()
		end
		return RLR_GetFullName(name, realm)
	else
		return nil
	end
end

function RLR_GetNameWithoutRealm(name)
	return (Ambiguate(name, 'short'))
end

function RLR_SendBroadcast(message, isHighPriority)
	SendChatMessage(RLR_SHORT_ADDON_NAME_PREFIX .. message, GetBroadcastChannel(isHighPriority))
end	

function RLR_SendWhisper(message, person)
	SendChatMessage(RLR_SHORT_ADDON_NAME_PREFIX .. message, 'WHISPER', nil, person)
end

function RLR_CreateEmptyTooltip(rows)
  local tip = CreateFrame('GameTooltip')
	local leftside = {}
	local rightside = {}
	local L, R
    for i = 1, rows do
        L, R = tip:CreateFontString(), tip:CreateFontString()
        L:SetFontObject(GameFontNormal)
        R:SetFontObject(GameFontNormal)
        tip:AddFontStrings(L, R)
        leftside[i] = L
		rightside[i] = R
    end
    tip.leftside = leftside
	tip.rightside = rightside
    return tip
end
