RaidLootRoller = LibStub("AceAddon-3.0"):NewAddon("RaidLootRoller", "AceTimer-3.0", "AceEvent-3.0")

-- indexes of array returned by GetFullItemInfo()
local FII_ITEM = "ITEM"									-- contains the actual item
local FII_NAME = "NAME"									-- return value 1 of Blizzard API call GetItemInfo()
local FII_LINK = "LINK"									-- return value 2 of Blizzard API call GetItemInfo()
local FII_QUALITY = "QUALITY"							-- return value 3 of Blizzard API call GetItemInfo()
local FII_BASE_ILVL = "BASE_ILVL"						-- return value 4 of Blizzard API call GetItemInfo()
local FII_REQUIRED_LEVEL = "REQUIRED_LEVEL"				-- return value 5 of Blizzard API call GetItemInfo()
local FII_TYPE = "TYPE"									-- return value 6 of Blizzard API call GetItemInfo()
local FII_SUB_TYPE = "SUB_TYPE"							-- return value 7 of Blizzard API call GetItemInfo()
local FII_MAX_STACK = "MAX_STACK"						-- return value 8 of Blizzard API call GetItemInfo()
local FII_ITEM_EQUIP_LOC = "ITEM_EQUIP_LOC"				-- return value 9 of Blizzard API call GetItemInfo()
local FII_TEXTURE = "TEXTURE"							-- return value 10 of Blizzard API call GetItemInfo()
local FII_VENDOR_PRICE = "VENDOR_PRICE"					-- return value 11 of Blizzard API call GetItemInfo()
local FII_CLASS = "CLASS"								-- return value 12 of Blizzard API call GetItemInfo()
local FII_SUB_CLASS = "SUB_CLASS"						-- return value 13 of Blizzard API call GetItemInfo()
local FII_BIND_TYPE = "BIND_TYPE"						-- return value 14 of Blizzard API call GetItemInfo()
local FII_EXPAC_ID = "EXPAC_ID"							-- return value 15 of Blizzard API call GetItemInfo()
local FII_ITEM_SET_ID = "ITEM_SET_ID"					-- return value 16 of Blizzard API call GetItemInfo()
local FII_IS_CRAFTING_REAGENT = "IS_CRAFTING_REAGENT"	-- return value 17 of Blizzard API call GetItemInfo()
local FII_IS_EQUIPPABLE = "IS_EQUIPPABLE"				-- true if the item is equippable, false otherwise
local FII_IS_RELIC = "IS_RELIC"							-- true if item is a relic, false otherwise
local FII_REAL_ILVL = "REAL_ILVL"						-- real ilvl, derived from tooltip
local FII_RELIC_TYPE = "RELIC_TYPE"						-- relic type, derived from tooltip
local FII_CLASSES = "CLASSES"							-- uppercase string of classes that can use the item (ex: tier); nil if item is not class-restricted

local TRADE_MESSAGE = 'TRADE'

local whisperedItems = {}  -- list of items we've whispered to people; index is character name-realm, content is item

local numOfQueuedRollItems = 0
local queuedRollOwners = {}
local queuedRollItems = {}

local currentRollOwner = nil
local currentRollItem = nil
local currentRolls = {}

local RLR_RANDOM_ROLL_RESULT_PATTERN = _G.RANDOM_ROLL_RESULT
	  RLR_RANDOM_ROLL_RESULT_PATTERN = RLR_RANDOM_ROLL_RESULT_PATTERN:gsub('%%s', '(.+)')
	  RLR_RANDOM_ROLL_RESULT_PATTERN = RLR_RANDOM_ROLL_RESULT_PATTERN:gsub('%%d %(%%d%-%%d%)', '(%%d+) %%((%%d+)%%-(%%d+)%%)')
	  RLR_RANDOM_ROLL_RESULT_PATTERN = '^' .. RLR_RANDOM_ROLL_RESULT_PATTERN .. '$'

local RLR_ITEM_LEVEL_PATTERN = _G.ITEM_LEVEL
RLR_ITEM_LEVEL_PATTERN = RLR_ITEM_LEVEL_PATTERN:gsub('%%d', '(%%d+)')  -- 'Item Level (%d+)'

local RLR_RELIC_TOOLTIP_TYPE_PATTERN = _G.RELIC_TOOLTIP_TYPE
RLR_RELIC_TOOLTIP_TYPE_PATTERN = RLR_RELIC_TOOLTIP_TYPE_PATTERN:gsub('%%s', '(.+)')  -- '(.+) Artifact Relic'

local RLR_CLASSES_ALLOWED_PATTERN = _G.ITEM_CLASSES_ALLOWED
	  RLR_CLASSES_ALLOWED_PATTERN = RLR_CLASSES_ALLOWED_PATTERN:gsub('%%s', '(.+)') 

function RaidLootRoller:OnInitialize()
	local defaults = {
		profile = {
			enabled = true,
			delay = 20,
			displayInstructions = false,
			monitorInstanceChat = false
		}
	}

	self.db = LibStub("AceDB-3.0"):New("RaidLootRollerDB", defaults, true)

	local optionsTable = {
		type = "group",
		args = {
			toggle = {
				order = 0,
				name = "Enable",
				desc = "Enables / disables the addon",
				type = "toggle",
				width = "full",
				set = function(info,val) self:ToggleEnableDisable(info, val) end,
				get = function(info) return self.db.profile.enabled end,
			},
			b1 = {
				order = 1,
				type = "header",
				name = "Options"
			},
			displayInstructions = {
				order = 2,
				name = "Show usage instructions in raid chat when a boss is killed (requires raid leader or assist).",
				type = "toggle",
				width = "full",
				set = function(info,val) self.db.profile.displayInstructions = val end,
				get = function(info) return self.db.profile.displayInstructions end,
			},
			monitorInstanceChat = {
				order = 3,
				name = "Respond to 'trade [Item]' messages in party/raid chat (requires leader or assist).",
				type = "toggle",
				width = "full",
				set = function(info,val) self:ToggleMonitorInstanceChat(info, val) end,
				get = function(info) return self.db.profile.monitorInstanceChat end,
			},
			delay = {
				order = 4,
				name = "Roll Timer",
				desc = "Set the amount of time a roll should take in seconds.",
				type = "range",
				width = "double",
				min = 15,
				max = 30,
				step = 5,
				set = function(info,val) self.db.profile.delay = val end,
				get = function(info) return self.db.profile.delay end
			}
		}
	}
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RaidLootRoller", optionsTable, {"rlr"})
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RaidLootRoller", "RaidLootRoller");

	if (self.db.profile.enabled) then
		print("<RLR> RaidLootRoller Enabled")
		self:RegisterEvent("CHAT_MSG_WHISPER", "WhisperReceivedEvent")
		self:RegisterEvent("ENCOUNTER_END", "EncounterEndedEvent")

		if (self.db.profile.monitorInstanceChat) then
			self:MonitorInstanceChat()
		end
	end
end

function RaidLootRoller:OnDisable()
end

function RaidLootRoller:ToggleEnableDisable(info, val)
	if (val) then
		if (self.db.profile.enabled == false) then
			print('<RLR> RaidLootRoller has been enabled.')
			self:RegisterEvent("CHAT_MSG_WHISPER", "WhisperReceivedEvent")
			self:RegisterEvent("ENCOUNTER_END", "EncounterEndedEvent")
			if (self.db.profile.monitorInstanceChat) then
				self:MonitorInstanceChat()
			end
		end
	else
		print('<RLR> RaidLootRoller has been disabled.')
		self:UnregisterEvent("CHAT_MSG_WHISPER", "WhisperReceivedEvent")
		self:UnregisterEvent("ENCOUNTER_END", "EncounterEndedEvent")
		self:UnmonitorInstanceChat()
	end
	self.db.profile.enabled = val
end

function RaidLootRoller:ToggleMonitorInstanceChat(info, val)
	if (val) then
		if (self.db.profile.monitorInstanceChat == false) then
			self:MonitorInstanceChat()
		end
	else
		self:UnmonitorInstanceChat()
	end
	self.db.profile.monitorInstanceChat = val
end

function RaidLootRoller:MonitorInstanceChat()
	self:RegisterEvent("CHAT_MSG_RAID", "ChatReceivedEvent")
	self:RegisterEvent("CHAT_MSG_RAID_LEADER", "ChatReceivedEvent")
	self:RegisterEvent("CHAT_MSG_PARTY", "ChatReceivedEvent")
	self:RegisterEvent("CHAT_MSG_PARTY_LEADER", "ChatReceivedEvent")
	self:RegisterEvent("CHAT_MSG_INSTANCE_CHAT", "ChatReceivedEvent")
	self:RegisterEvent("CHAT_MSG_INSTANCE_CHAT_LEADER", "ChatReceivedEvent")
end

function RaidLootRoller:UnmonitorInstanceChat()
	self:UnregisterEvent("CHAT_MSG_RAID", "ChatReceivedEvent")
	self:UnregisterEvent("CHAT_MSG_RAID_LEADER", "ChatReceivedEvent")
	self:UnregisterEvent("CHAT_MSG_PARTY", "ChatReceivedEvent")
	self:UnregisterEvent("CHAT_MSG_PARTY_LEADER", "ChatReceivedEvent")
	self:UnregisterEvent("CHAT_MSG_INSTANCE_CHAT", "ChatReceivedEvent")
	self:UnregisterEvent("CHAT_MSG_INSTANCE_CHAT_LEADER", "ChatReceivedEvent")
end

function RaidLootRoller:EncounterEndedEvent(event, ...)
	local encounterID, encounterName, difficultyID, groupSize, success = ...
	if (success and self.db.profile.enabled and self.db.profile.displayInstructions) then
		inInstance, instanceType = IsInInstance()
		if instanceType == "raid" and (UnitIsGroupLeader("player") or UnitIsGroupAssistant("player")) then
			RaidLootRoller:ScheduleTimer("BroadcastInstructions", 5)
		end
	end 
end

function RaidLootRoller:BroadcastInstructions()
	RLR_SendBroadcast('RaidLootRoller - If you received any loot that you do not need and are able '
		.. 'to trade please whisper me the word "trade" followed by the item to initiate a roll.', false)
end

function RaidLootRoller:WhisperReceivedEvent(event, ...)
  local message, sender = ...
  self:ProcessWhisper(message, sender)
end

function RaidLootRoller:ChatReceivedEvent(event, ...)
	-- only offer rolls shown in raid/party chat if the user is a group leader or has assist in raid
	if UnitIsGroupLeader("player") or UnitIsGroupAssistant("player") then
		local message, sender = ...
		self:ProcessWhisper(message, sender)
	end
end

function RaidLootRoller:ProcessWhisper(message, sender) 
  if not string.find(sender, '-') then
    sender = RLR_GetUnitNameWithRealm(sender)
  end

  -- if the person whispered 'trade [item]' or '[item] trade', then add the item to the array so we can process it
	local _, _, whisperedItem = string.find(message, 'trade (|.+|r)')
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, 'trade(|.+|r)')
  end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, 'Trade (|.+|r)')
	end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, 'Trade(|.+|r)')
  end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, 'TRADE (|.+|r)')
	end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, 'TRADE(|.+|r)')
  end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, '(|.+|r) trade')
	end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, '(|.+|r)trade')
  end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, '(|.+|r) Trade')
	end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, '(|.+|r)Trade')
  end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, '(|.+|r) TRADE')
	end
  if whisperedItem == nil then
    _, _, whisperedItem = string.find(message, '(|.+|r)TRADE')
  end
  if whisperedItem ~= nil then
    whisperedItems[sender] = whisperedItem
  end

  message = string.upper(message)

  if whisperedItem ~= nil or message == TRADE_MESSAGE or message == '\'' .. TRADE_MESSAGE .. '\'' then
    if whisperedItems[sender] ~= nil then
      local item = whisperedItems[sender]

      whisperedItems[sender] = nil
      self:QueueItem(sender, item)

      -- if we're still rolling for another item, let the person know their item is queued
      if currentRollItem ~= nil then
        RLR_SendWhisper('Thank you! ' .. item .. ' will be rolled for after current rolls are done.', sender)
      else 
        RLR_SendWhisper('Thank you! ' .. item .. ' is being rolled for now.', sender)
        self:AskForRolls()
      end
		end
  end
 
end

function RaidLootRoller:QueueItem(sender, item)
	queuedRollOwners[numOfQueuedRollItems] = sender
	queuedRollItems[numOfQueuedRollItems] = item
	numOfQueuedRollItems = numOfQueuedRollItems + 1
end

function RaidLootRoller:AskForRolls()
  if currentRollItem == nil and numOfQueuedRollItems > 0 then

    -- start watching for roll events
    self:RegisterEvent("CHAT_MSG_SYSTEM", "RollReceivedEvent")
    
		currentRollOwner = queuedRollOwners[numOfQueuedRollItems - 1]
		currentRollItem = queuedRollItems[numOfQueuedRollItems - 1]
    numOfQueuedRollItems = numOfQueuedRollItems - 1

    -- try and get details about the item up for trade
    local description = ""
    if IsEquippableItem(currentRollItem) then
      local fullItemInfo = self:GetFullItemInfo(currentRollItem)
      description = " ("
      if fullItemInfo[FII_REAL_ILVL] ~= nil then
        description = description .. fullItemInfo[FII_REAL_ILVL] .. " "
      end
      if fullItemInfo[FII_IS_RELIC] and fullItemInfo[FII_RELIC_TYPE] ~= nil then
        description = description .. fullItemInfo[FII_RELIC_TYPE] .. " Relic"
      else
        if fullItemInfo[FII_CLASS] == LE_ITEM_CLASS_ARMOR then
          if fullItemInfo[FII_SUB_CLASS] == LE_ITEM_ARMOR_GENERIC or fullItemInfo[FII_ITEM_EQUIP_LOC] == "INVTYPE_CLOAK" then
            description = description .. _G[fullItemInfo[FII_ITEM_EQUIP_LOC]]
          else
            description = description .. fullItemInfo[FII_SUB_TYPE] .. " " .. _G[fullItemInfo[FII_ITEM_EQUIP_LOC]]
          end
        else
          description = description .. fullItemInfo[FII_SUB_TYPE]
        end
        
      end
      description = description .. ")"
    end


		RLR_SendBroadcast('Roll for ' .. currentRollItem .. description .. ' from ' .. RLR_GetNameWithoutRealm(currentRollOwner), true)
	
		RaidLootRoller:ScheduleTimer("tenSecondsRemaining", self.db.profile.delay - 10)
		RaidLootRoller:ScheduleTimer("fiveSecondsRemaining", self.db.profile.delay - 5)
    RaidLootRoller:ScheduleTimer("zeroSecondsRemaining", self.db.profile.delay)
    RaidLootRoller:ScheduleTimer("EndRolls", self.db.profile.delay + 1)
	end
end

function RaidLootRoller:tenSecondsRemaining()
  RLR_SendBroadcast('10 seconds remaining to roll for ' .. currentRollItem, true)
end

function RaidLootRoller:fiveSecondsRemaining()
  RLR_SendBroadcast('5 seconds remaining to roll for ' .. currentRollItem, true)
end

function RaidLootRoller:zeroSecondsRemaining()
  RLR_SendBroadcast('Rolls ended for ' .. currentRollItem, false)
end

function RaidLootRoller:RollReceivedEvent(event, ...)
	if currentRollItem ~= nil then
		local message = select(1, ...)
		if message then
			-- local name, roll, minRoll, maxRoll = message:match('^(.+) rolls (%d+) %((%d+)%-(%d+)%)$')
			local name, roll, minRoll, maxRoll = message:match(RLR_RANDOM_ROLL_RESULT_PATTERN)
      if name then
        local fullname = RLR_GetUnitNameWithRealm(name)
				if minRoll ~= '1' or maxRoll ~= '100' then
          RLR_SendBroadcast(name .. ' rolled ' .. minRoll .. ' - ' .. maxRoll .. '; roll ignored', false)
				elseif currentRolls[name] ~= nil then
          RLR_SendBroadcast(name .. ' rolled multiple times; only the first roll of ' .. currentRolls[name] .. ' counts', false)
        else
					currentRolls[name] = roll
				end
			end		
    end
  end
end

function RaidLootRoller:EndRolls()
  -- stop watching for roll events
  self:UnregisterEvent("CHAT_MSG_SYSTEM", "RollReceivedEvent")


	-- determine the winner(s)
	local winners = nil
	local highRoll = 0
	for name, roll in pairs(currentRolls) do
		roll = tonumber(roll)
		if winners == nil or roll > highRoll then
			winners = { name }
			highRoll = roll
		elseif winners ~= nil and roll == highRoll then
			table.insert(winners, name)
		end
  end

	-- notify everyone of the results of the rolls
	if winners ~= nil then
    if #winners > 1 then
			RLR_SendBroadcast(self:GetNames(winners) .. ' tied for ' .. currentRollItem .. ' from ' .. RLR_GetNameWithoutRealm(currentRollOwner) .. ' with a ' .. highRoll, true)
    else
      RLR_SendBroadcast(winners[1] .. ' won ' .. currentRollItem .. ' from ' .. RLR_GetNameWithoutRealm(currentRollOwner) .. ' with a ' .. highRoll, true)
      -- wisper winner
      RLR_SendWhisper('You won ' .. currentRollItem .. '! Please open trade with ' .. currentRollOwner .. '.', winners[1])
      -- wisper loot owner
      RLR_SendWhisper(winners[1] .. ' won the roll for your ' .. currentRollItem .. ' with a ' .. highRoll, currentRollOwner)
		end
  else
	  RLR_SendBroadcast('Nobody rolled for ' .. currentRollItem .. ' from ' .. RLR_GetNameWithoutRealm(currentRollOwner), true)
  end

  self:ClearRolls()
  
  -- if there's another roll to do, let's delay a few seconds before starting the next roll
  if (numOfQueuedRollItems > 0) then
    RaidLootRoller:ScheduleTimer("AskForRolls", 5)
  end

end

function RaidLootRoller:ClearRolls()
	currentRollOwner = nil
	currentRollItem = nil
	currentRolls = {}
	queuedRollOwners[numOfQueuedRollItems] = nil
	queuedRollItems[numOfQueuedRollItems] = nil
end

-- returns the names from the given array, with 'and others' if array size > limit
function RaidLootRoller:GetNames(namelist, limit)
	local names = ''
	if namelist ~= nil then
		if limit == nil then  -- no limit; show all names
			limit = #namelist
		end
		if namelist[1] ~= nil then
			-- sort the array by ilvl first
			local sortedNamelist = namelist
			if #namelist > 1 then
				local copiedNamelist = ShallowCopy(namelist)  -- we will destroy elements in the list while sorting, so copy it
				sortedNamelist = {}
				local lowestILVL
				local lowestIndex
				local ilvl
				local i = 1
				local size = #copiedNamelist
				while i <= size do
					lowestILVL = 1000000
					lowestIndex = 1  -- we could be sorting a list without ilvls, in which case just keep the same order
					for j = 1, #copiedNamelist do
						if copiedNamelist[j] ~= nil then
							ilvl = string.match(copiedNamelist[j], '(%d+)')
							if ilvl ~= nil then
								ilvl = tonumber(ilvl)
								if ilvl < lowestILVL then
									lowestILVL = ilvl
									lowestIndex = j
								end
							end
						end
					end
					table.insert(sortedNamelist, table.remove(copiedNamelist, lowestIndex))
					i = i + 1
				end
			end
		
			names = sortedNamelist[1]
			local maxnames = min(#sortedNamelist, limit)
			for i = 2, maxnames do
				if #sortedNamelist == 2 then
					names = names .. ' '
				else
					names = names .. ', '
				end
				if i == #sortedNamelist then -- last person
					names = names .. 'and '
				end
				names = names .. sortedNamelist[i]
			end
			if #sortedNamelist > limit then
				names = names .. ', and others'
			end
		end
	end
	return names
end

function RaidLootRoller:GetItemFromQueueByPlayer(player)
	for key, value in pairs(queuedRollOwners) do
		if value == player then
			return queuedRollItems[key]
		end
	end
	return nil
end

function RaidLootRoller:GetFullItemInfo(item)
	fullItemInfo = {}
	if item ~= nil then
		fullItemInfo[FII_ITEM] = item
		
		-- determine the basic values from the Blizzard GetItemInfo() API call
		fullItemInfo[FII_NAME],
			fullItemInfo[FII_LINK],
			fullItemInfo[FII_QUALITY],
			fullItemInfo[FII_BASE_ILVL],
			fullItemInfo[FII_REQUIRED_LEVEL],
			fullItemInfo[FII_TYPE],
			fullItemInfo[FII_SUB_TYPE],
			fullItemInfo[FII_MAX_STACK],
			fullItemInfo[FII_ITEM_EQUIP_LOC],
			fullItemInfo[FII_TEXTURE],
			fullItemInfo[FII_VENDOR_PRICE],
			fullItemInfo[FII_CLASS],
			fullItemInfo[FII_SUB_CLASS],
			fullItemInfo[FII_BIND_TYPE],
			fullItemInfo[FII_EXPAC_ID],
			fullItemInfo[FII_ITEM_SET_ID],
			fullItemInfo[FII_IS_CRAFTING_REAGENT]
			= GetItemInfo(item)

		-- determine whether the item is equippable & whether it is a relic
		fullItemInfo[FII_IS_EQUIPPABLE] = IsEquippableItem(item)
		fullItemInfo[FII_IS_RELIC] = fullItemInfo[FII_CLASS] == LE_ITEM_CLASS_GEM and fullItemInfo[FII_SUB_CLASS] == LE_ITEM_ARMOR_RELIC

		-- we only need to determine other values if it's an equippable item or a relic
		if fullItemInfo[FII_IS_EQUIPPABLE] or fullItemInfo[FII_IS_RELIC] then

			-- set up the tooltip to determine values that aren't returned via GetItemInfo()
			local rows = 30
			if fullItemInfo[FII_IS_RELIC] then
				rows = 6  -- if it's a relic, we only need to inspect the first 6 rows
			end
			tooltip = tooltip or RLR_CreateEmptyTooltip(30)
			tooltip:SetOwner(UIParent, 'ANCHOR_NONE')
			tooltip:ClearLines()
			tooltip:SetHyperlink(item)
			local t
			local index

			-- determine the real iLVL
			local realILVL = nil
			t = tooltip.leftside[2]:GetText()
			if t ~= nil then
				realILVL = t:match(RLR_ITEM_LEVEL_PATTERN)
			end

			if realILVL == nil then  -- ilvl can be in the 2nd or 3rd line dependng on the tooltip; if we didn't find it in 2nd, try 3rd
				t = tooltip.leftside[3]:GetText()
				if t ~= nil then
					realILVL = t:match(RLR_ITEM_LEVEL_PATTERN)
				end
			end
			if realILVL == nil then  -- if we still couldn't find it (shouldn't happen), just use the ilvl we got from GetItemInfo()
				realILVL = fullItemInfo[FII_BASE_ILVL]
			end
			fullItemInfo[FII_REAL_ILVL] = tonumber(realILVL)

			-- if the item is a relic, determine the relic type
			local relicType = nil
			if fullItemInfo[FII_IS_RELIC] then
				index = 1
				while not relicType and tooltip.leftside[index] do
					t = tooltip.leftside[index]:GetText()
					if t ~= nil then
						relicType = t:match(RLR_RELIC_TOOLTIP_TYPE_PATTERN)				
					end
					index = index + 1
				end
			end
			fullItemInfo[FII_RELIC_TYPE] = relicType

			-- if the item is restricted to certain classes, determine which ones
			local classes = nil
			index = 1
			while not classes and tooltip.leftside[index] do
				t = tooltip.leftside[index]:GetText()
				if t ~= nil then
					classes = t:match(RLR_CLASSES_ALLOWED_PATTERN)
				end
				index = index + 1
			end
			if classes ~= nil then
				classes = string.upper(classes)
				classes = string.gsub(classes, " ", "")  -- remove space for DEMON HUNTER, DEATH KNIGHT
			end
			fullItemInfo[FII_CLASSES] = classes

			-- hide the tooltip now that we're done with it (is this really necessary?)
			tooltip:Hide()
		end
  end

	return fullItemInfo
end
